package hu.kozadat.oaiaggregator.persistence;

import hu.kozadat.oaiaggregator.domain.Record;
import hu.kozadat.oaiaggregator.domain.Schema;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;

/**
 * Created by tnagy on 2015.10.14..
 */
@Repository
public class KozadatDAOSapDBImpl implements KozadatDAO {

    private static final Logger LOGGER = LoggerFactory.getLogger(KozadatDAOSapDBImpl.class);

    private static final String STMT_RECORDS_WITH_SCHEMAS = "select " +
            "r.id, r.oai_id, r.dp_id, r.xml, r.state, r.mdc_schema, r.modified, " +
            "r.invalid_date, r.harvest_id, r.unit_description_id, " +
            "s.id as s_id, s.type, s.schema_id, s.schema_loc, s.schema_class, " +
            "s.schema_name, s.schema_desc, s.schema_prefix, " +
            "s.root_element_name, s.parent_id, s.created as s_created, s.modified s_modified " +
            "from xmls r " +
            "left outer join schemas s on r.mdc_schema = s.id";

    @Autowired
    private JdbcTemplate jdbcTemplate;

    private RowMapper<Record> recordRowMapper = new RowMapper<Record>() {
        @Override
        public Record mapRow(ResultSet rs, int rowNum) throws SQLException {
            Record record = extractRecord(rs);
            record.setSchema(extractSchema(rs));
            return record;
        }
    };
    private RowMapper<Schema> schemaRowMapper = new RowMapper<Schema>() {
        @Override
        public Schema mapRow(ResultSet rs, int rowNum) throws SQLException {
            return extractSchema(rs);
        }
    };

    @Override
    public String getEarliestDateString() {
        return jdbcTemplate.queryForObject("select modified from xmls order by modified asc limit 1", String.class);
    }

    @Override
    public List<Schema> findMetadataSchemas() {
        List<Schema> schemas = jdbcTemplate.query("select " +
                "s.id as s_id, s.type, s.schema_id, s.schema_loc, s.schema_class, " +
                "s.schema_name, s.schema_desc, s.schema_prefix, s.root_element_name, " +
                "s.parent_id, s.created as s_created, s.modified as s_modified, s.xml " +
                "from schemas s where s.type = ?", new Object[]{"S"}, schemaRowMapper);
        return schemas;
    }

    @Override
    public Record findMetadataRecord(String identifier, String metadataPrefix) {
        StringBuilder statement = new StringBuilder(STMT_RECORDS_WITH_SCHEMAS);
        List<Object> params = new LinkedList<>();
        if (identifier != null) {
            statement.append(" where ");
            statement.append("r.oai_id = ?");
            params.add(identifier);
        }
        if (metadataPrefix != null) {
            if (params.isEmpty()) {
                statement.append(" where ");
            } else {
                statement.append(" and ");
            }
            statement.append(" s.schema_prefix = ?");
            params.add(metadataPrefix);
        }

        return jdbcTemplate.queryForObject(statement.toString(), params.toArray(), recordRowMapper);
    }

    @Override
    public List<Long> findMetadataRecordIds(String metadataPrefix, String fromDate, String untilDate) {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("findMetadataRecordIds(): CALLED, params: metadataPrefix={}, fromDate={}, untilDate={}", metadataPrefix, fromDate, untilDate);
        }
        StringBuilder statement = new StringBuilder("select r.id from xmls r left outer join schemas s on r.mdc_schema = s.id");
        List<Object> params = new LinkedList<>();
        if (metadataPrefix != null) {
            statement.append(" where ");
            statement.append("s.schema_prefix = ?");
            params.add(metadataPrefix);
        }
        if (fromDate != null) {
            if (params.isEmpty()) {
                statement.append(" where ");
            } else {
                statement.append(" and ");
            }
            statement.append("r.modified >= ?");
            params.add(fromDate);

        }
        if (untilDate != null) {
            if (params.isEmpty()) {
                statement.append(" where ");
            } else {
                statement.append(" and ");
            }
            statement.append("r.modified < ?");
            params.add(untilDate);

        }

        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("findMetadataRecordIds(): statement={}, params=", statement.toString(), params.toArray());
        }
        return jdbcTemplate.queryForList(statement.toString(), params.toArray(), Long.class);
    }

    @Override
    public List<Record> findMetadataRecords(List<Long> identifiers) {
        StringBuilder statement = new StringBuilder(STMT_RECORDS_WITH_SCHEMAS);
        List<Object> params = new LinkedList<>();
        if (identifiers != null && !identifiers.isEmpty()) {
            statement.append(" where r.id in (");
            for (int i = 0; i < identifiers.size(); i++) {
                params.add(identifiers.get(i));
                statement.append("?");
                if (i < identifiers.size() - 1) {
                    statement.append(",");
                }
            }
            statement.append(")");
        }
        return jdbcTemplate.query(statement.toString(), params.toArray(), recordRowMapper);
    }

    private Schema extractSchema(ResultSet rs) throws SQLException {
        Schema schema = new Schema();
        schema.setId(rs.getLong("s_id"));
        schema.setType(rs.getString("type"));
        schema.setSchemaId(rs.getString("schema_id"));
        schema.setSchemaLocation(rs.getString("schema_loc"));
        schema.setSchemaClass(rs.getLong("schema_class"));
        schema.setSchemaName(rs.getString("schema_name"));
        schema.setSchemaDescription(rs.getString("schema_desc"));
        schema.setSchemaPrefix(rs.getString("schema_prefix"));
        if (rs.wasNull()) {
            // tnagy set empty string instead of null for prefix
            schema.setSchemaPrefix("");
        }
        schema.setRootElementName(rs.getString("root_element_name"));
        schema.setParentId(rs.getLong("parent_id"));
        schema.setCreated(rs.getString("s_created"));
        schema.setModified(rs.getString("s_modified"));
        return schema;
    }

    private Record extractRecord(ResultSet rs) throws SQLException {
        Record record = new Record();
        record.setId(rs.getLong("id"));
        record.setOaiId(rs.getString("oai_id"));
        record.setDataProviderId(rs.getLong("dp_id"));
        record.setXml(rs.getBytes("xml"));
        record.setState(rs.getString("state"));
        record.setSchemaId(rs.getLong("mdc_schema"));
        record.setModified(rs.getString("modified"));
        record.setInvalidDate(rs.getString("invalid_date"));
        record.setHarvestId(rs.getLong("harvest_id"));
        record.setUnitDescriptionId(rs.getLong("unit_description_id"));
        return record;
    }
}
